package order

import (
	"applet/app/admin/md"
	"applet/app/db"
	"applet/app/db/model"
	"applet/app/e"
	"applet/app/svc"
	"applet/app/utils"
	"encoding/json"
	"fmt"
	"github.com/360EntSecGroup-Skylar/excelize"
	"github.com/gin-gonic/gin"
	"github.com/tidwall/gjson"
	"strconv"
)

func OrderList(c *gin.Context) {
	var args map[string]string
	if err := c.ShouldBindJSON(&args); err != nil {
		e.OutErr(c, e.ERR_INVALID_ARGS, err)
		return
	}
	eg := db.Db
	list, total := db.GetOrderList(eg, args)
	data := make([]map[string]string, 0)
	if list != nil {
		stateArr := []string{"待制作", "制作中", "制作完成", "烘焙完成", "分拣完成", "已取消"}
		enterpriseIds := make([]int, 0)
		for _, v := range *list {
			enterpriseIds = append(enterpriseIds, v.EnterpriseId)
		}
		enterpriseMap := db.GetEnterpriseMore(eg, enterpriseIds)
		for _, v := range *list {
			v1 := JudgePackageOrdOrdState(&v)
			isCanCancel := "1"
			if v1.State > 0 {
				isCanCancel = "0"
			}
			enterpriseName := ""
			_, ok := enterpriseMap[v.EnterpriseId]
			if ok {
				enterpriseName = enterpriseMap[v.EnterpriseId].Name
			}
			var tmp = map[string]string{
				"buy_phone":       v1.BuyPhone,
				"oid":             utils.Int64ToStr(v1.Oid),
				"id":              utils.IntToStr(v1.Id),
				"amount":          v1.Amount,
				"state_str":       stateArr[v1.State],
				"state":           utils.IntToStr(v1.State),
				"time":            gjson.Get(v1.BuyInfo, "time").String(),
				"date":            gjson.Get(v1.BuyInfo, "date").String(),
				"create_at":       v1.CreateAt.Format("2006-01-02 15:04:05"),
				"is_can_cancel":   isCanCancel,
				"enterprise_name": enterpriseName,
			}
			data = append(data, tmp)
		}
	}
	state := []map[string]string{
		{"name": "待制作", "value": "0"},
		{"name": "制作中", "value": "1"},
		{"name": "烘焙中", "value": "2"},
		{"name": "分拣中", "value": "3"},
		{"name": "已完成", "value": "4"},
		{"name": "已取消", "value": "5"},
	}
	res := map[string]interface{}{
		"total": total,
		"list":  data,
		"state": state,
	}
	e.OutSuc(c, res, nil)
	return
}

func OrderExport(c *gin.Context) {
	var args map[string]string
	if err := c.ShouldBindJSON(&args); err != nil {
		e.OutErr(c, e.ERR_INVALID_ARGS, err)
		return
	}
	eg := db.Db
	list, _ := db.GetOrderListExport(eg, args)

	titleList := []string{"订单编号", "金额", "手机号", "企业名称", "学校名称", "状态", "送达时间", "订单时间"}
	xlsx := excelize.NewFile()
	xlsx.SetSheetRow("Sheet1", "A1", &titleList)
	style, err := xlsx.NewStyle(`{"fill":{"type":"pattern","color":["FFFF00"],"pattern":1}, "alignment":{"horizontal":"center"}}`)
	if err != nil {
		e.OutErr(c, e.ERR, err.Error())
		return
	}
	xlsx.SetCellStyle("Sheet1", "A1", "M1", style)
	xlsx.SetColWidth("Sheet1", "A", "A", 35)
	xlsx.SetColWidth("Sheet1", "B", "B", 30)
	xlsx.SetColWidth("Sheet1", "C", "E", 15)
	xlsx.SetColWidth("Sheet1", "F", "I", 10)
	xlsx.SetColWidth("Sheet1", "J", "J", 20)
	xlsx.SetColWidth("Sheet1", "K", "K", 10)
	xlsx.SetColWidth("Sheet1", "L", "L", 25)
	xlsx.SetColWidth("Sheet1", "M", "M", 15)
	//表头被第一行用了，只能从第二行开始
	j := 2

	if list != nil {
		stateArr := []string{"待制作", "制作中", "制作完成", "烘焙完成", "分拣完成", "已取消"}
		enterpriseIds := make([]int, 0)
		for _, v := range *list {
			enterpriseIds = append(enterpriseIds, v.EnterpriseId)
		}
		enterpriseMap := db.GetEnterpriseMore(eg, enterpriseIds)
		for _, v := range *list {
			v1 := JudgePackageOrdOrdState(&v)
			enterpriseName := ""
			_, ok := enterpriseMap[v.EnterpriseId]
			if ok {
				enterpriseName = enterpriseMap[v.EnterpriseId].Name
			}

			xlsx.SetSheetRow("Sheet1", "A"+strconv.Itoa(j), &[]interface{}{
				utils.Int64ToStr(v1.Oid), v1.Amount, v1.BuyPhone, enterpriseName, stateArr[v1.State],
				gjson.Get(v1.BuyInfo, "date").String() + " " + gjson.Get(v1.BuyInfo, "time").String(), v1.CreateAt.Format("2006-01-02 15:04:05"),
			})
			j++
		}
	}

	c.Header("Content-Type", "application/octet-stream")
	c.Header("Content-Disposition", "attachment; filename="+"订单记录"+".xlsx")
	c.Header("Content-Transfer-Encoding", "binary")
	//回写到web 流媒体 形成下载
	_ = xlsx.Write(c.Writer)
	return
}

// JudgePackageOrdOrdState 判断订单状态
func JudgePackageOrdOrdState(ord *model.Order) *model.Order {
	if ord.State == 5 {
		return ord
	}
	var ordState, oldOrdState int
	oldOrdState = ord.State
	ordState = oldOrdState
	//全部订单
	countAll, err := db.Db.Where("oid =?", ord.Oid).Count(&model.OrderGoods{})
	if err != nil {
		return ord
	}
	//1、判断是否有 `制作中` 有一个就是制作中
	count1, err := db.Db.Where("oid =?", ord.Oid).And("state =?", 1).Count(&model.OrderGoods{})
	if err != nil {
		return ord
	}
	if count1 > 0 {
		ordState = 1
	}
	//3、判断是否有 `烘焙中` 要全部制作完成
	count2, err := db.Db.Where("oid =?", ord.Oid).And("state =?", 2).Count(&model.OrderGoods{})
	if err != nil {
		return ord
	}
	if count2 > 0 && count2 == countAll {
		ordState = 2
	}
	//3、判断是否有 `分拣中` 要全部烘焙完成
	count3, err := db.Db.Where("oid =?", ord.Oid).And("state =?", 3).Count(&model.OrderGoods{})
	if err != nil {
		return ord
	}
	if count3 > 0 && count3 == countAll {
		ordState = 3
	}
	//4、判断是否有 `已完成` 要全部已完成
	count4, err := db.Db.Where("oid =?", ord.Oid).And("state =?", 4).Count(&model.OrderGoods{})
	if err != nil {
		return ord
	}
	if count4 > 0 && count4 == countAll {
		ordState = 4
	}
	if ordState != oldOrdState {
		ord.State = ordState
		_, err2 := db.Db.Where("oid=?", ord.Oid).Cols("state").Update(&ord)
		if err2 != nil {
			return ord
		}
	}
	return ord
}
func OrderTotal(c *gin.Context) {
	var args map[string]string
	if err := c.ShouldBindJSON(&args); err != nil {
		e.OutErr(c, e.ERR_INVALID_ARGS, err)
		return
	}
	eg := db.Db
	where := "1=1"

	if args["phone"] != "" {
		where += " and buy_phone like '%" + args["phone"] + "%'"
	}
	if args["oid"] != "" {
		where += " and oid like '%" + args["oid"] + "%'"
	}
	if args["goods_title"] != "" {
		var orderGoods []model.OrderGoods
		eg.Where("goods_title like ?", "%"+args["goods_title"]+"%").Find(&orderGoods)
		oids := "-1"
		for _, v := range orderGoods {
			oids += "," + utils.Int64ToStr(v.Oid)
		}
		where += " and oid in(" + oids + ")"
	}
	sql := "select SUM(IF(state!=5,amount,0)) as amount,SUM(IF(state!=5,1,0)) as alls,SUM(IF(state=0,1,0)) as wait_do,SUM(IF(state=1,1,0)) as dos,SUM(IF(state=2,1,0)) as baking,SUM(IF(state=3,1,0)) as sorting,SUM(IF(state=4,1,0)) as success,SUM(IF(state=5,1,0)) as cancel from `order` where %s"
	sql = fmt.Sprintf(sql, where)
	nativeString, _ := db.QueryNativeString(eg, sql)
	var res = make([]string, 0)
	for _, v := range nativeString {
		res = append(res, "订单总金额(元):"+v["amount"])
		res = append(res, "总订单数量:"+v["alls"])
		res = append(res, "待制作订单数:"+v["wait_do"])
		res = append(res, "待烘焙订单数:"+v["dos"])
		res = append(res, "已完成订单数:"+v["success"])
		res = append(res, "已取消订单数:"+v["cancel"])
	}
	e.OutSuc(c, res, nil)
	return
}

func OrderCancel(c *gin.Context) {
	var args map[string]string
	if err := c.ShouldBindJSON(&args); err != nil {
		e.OutErr(c, e.ERR_INVALID_ARGS, err)
		return
	}
	eg := db.Db
	order := db.GetOrderById(eg, args["oid"])
	if order == nil {
		e.OutErr(c, 400, e.NewErr(400, "订单不存在"))
		return
	}
	order = JudgePackageOrdOrdState(order)
	if order.State == 5 {
		e.OutErr(c, 400, e.NewErr(400, "订单已取消"))
		return
	}
	if order.State > 0 {
		e.OutErr(c, 400, e.NewErr(400, "订单制作中,不能取消"))
		return
	}
	order.State = 5
	eg.Where("oid=?", order.Oid).Cols("state").Update(order)
	eg.Where("oid=?", order.Oid).Cols("state").Update(&model.OrderGoods{State: 5})
	e.OutSuc(c, "success", nil)
	return
}

func OrderDetail(c *gin.Context) {
	var args map[string]string
	if err := c.ShouldBindJSON(&args); err != nil {
		e.OutErr(c, e.ERR_INVALID_ARGS, err)
		return
	}
	eg := db.Db
	order := db.GetOrderById(eg, args["oid"])
	if order == nil {
		e.OutErr(c, 400, e.NewErr(400, "订单不存在"))
		return
	}
	order = JudgePackageOrdOrdState(order)
	var enterpriseData model.Enterprise
	eg.Where("id=?", order.EnterpriseId).Get(&enterpriseData)
	stateArr := []string{"待制作", "制作中", "烘焙中", "分拣中", "已完成", "已取消"}
	enterpriseName := enterpriseData.Name

	//订单商品
	var orderGoods []model.OrderGoods
	eg.Where("oid=?", args["oid"]).Find(&orderGoods)
	skuIds := make([]int64, 0)
	gids := make([]int64, 0)

	for _, v := range orderGoods {
		skuIds = append(skuIds, v.SkuId)
		gids = append(gids, v.GoodsId)
	}
	//查商品和sku
	goodsMap := db.GetGoodsMore(eg, gids)
	goodsInfo := make([]map[string]interface{}, 0)
	scheme, host, subDomain, moreSubDomain := svc.ImageBucketNew(c)
	var tmpMap interface{}
	var skus []model.Sku
	eg.Table("sku").In("goods_id", gids).Find(&skus)
	for _, v := range orderGoods {
		skuList := make([]model.Sku, 0)
		for _, skuItem := range skus {
			if skuItem.GoodsId == v.GoodsId {
				skuList = append(skuList, skuItem)
			}
		}
		tmp := map[string]interface{}{
			"goods_img":   "",
			"id":          utils.IntToStr(v.Id),
			"sku_id":      utils.Int64ToStr(v.SkuId),
			"goods_id":    utils.Int64ToStr(v.GoodsId),
			"num":         utils.IntToStr(v.Num),
			"amount":      utils.Float64ToStr(utils.StrToFloat64(v.Price) * float64(v.Num)),
			"sku_str":     "",
			"spe":         "",
			"sku_list":    skuList,
			"goods_title": v.GoodsTitle,
		}

		skuData := make([]md.Sku, 0)
		json.Unmarshal([]byte(v.Sku), &skuData)
		skuStr := ""
		for _, v1 := range skuData {
			if skuStr != "" {
				skuStr += ";"
			}
			skuStr += v1.Value
		}
		tmp["sku_str"] = skuStr

		// spe
		var spe []map[string]interface{}
		var speTmp []map[string]interface{}
		utils.Unserialize([]byte(goodsMap[v.GoodsId].Spe), &spe)
		for _, itemSpe := range spe {
			tmp1 := make(map[string]interface{})
			tmp1["type"] = "text"
			tmp1["name"] = itemSpe["name"]
			subTmp := make([]map[string]interface{}, 0)
			for i, v2 := range itemSpe["values"].([]interface{}) {
				subTmp = append(subTmp, map[string]interface{}{
					"key":  i,
					"name": v2,
				})
			}
			tmp1["values"] = subTmp
			speTmp = append(speTmp, tmp1)
		}
		tmp["spe"] = speTmp
		// 商品主图
		if goodsMap[v.GoodsId].ImageList != "" {
			utils.Unserialize([]byte(goodsMap[v.GoodsId].ImageList), &tmpMap)
			var imageListUrlTmp []string
			for _, v := range tmpMap.([]interface{}) {
				imageListUrlTmp = append(imageListUrlTmp, svc.ImageFormatWithBucketNew(scheme, host, subDomain, moreSubDomain, v.(string)))
			}
			if imageListUrlTmp != nil {
				tmp["goods_img"] = imageListUrlTmp[0]
			}
		}
		goodsInfo = append(goodsInfo, tmp)
	}

	var res = map[string]interface{}{
		"buy_phone":       order.BuyPhone,
		"oid":             utils.Int64ToStr(order.Oid),
		"id":              utils.IntToStr(order.Id),
		"amount":          order.Amount,
		"state_str":       stateArr[order.State],
		"state":           utils.IntToStr(order.State),
		"date":            gjson.Get(order.BuyInfo, "date").String(),
		"buy_name":        gjson.Get(order.BuyInfo, "name").String(),
		"address":         gjson.Get(order.BuyInfo, "address").String(),
		"time":            gjson.Get(order.BuyInfo, "time").String(),
		"create_at":       order.CreateAt.Format("2006-01-02 15:04:05"),
		"enterprise_name": enterpriseName,
		"goods_info":      goodsInfo,
	}
	e.OutSuc(c, res, nil)
	return
}

func OrderOutput(c *gin.Context) {
	var args map[string]string
	if err := c.ShouldBindJSON(&args); err != nil {
		e.OutErr(c, e.ERR_INVALID_ARGS, err)
		return
	}
	eg := db.Db
	list := db.GetOrderListOutput(eg, args)

	name := "订单_" + args["page"]
	//写入数据
	data := map[string]string{
		"A1": "购买人手机号",
		"B1": "学校名称",
		"C1": "订单编号",
		"D1": "总金额",
		"E1": "订单状态",
		"F1": "送达时间",
		"G1": "下单时间",
	}
	if list != nil {
		stateArr := []string{"待制作", "制作中", "制作完成", "烘焙完成", "分拣完成", "已取消"}
		enterpriseIds := make([]int, 0)
		for _, v := range *list {
			enterpriseIds = append(enterpriseIds, v.EnterpriseId)
		}
		enterpriseMap := db.GetEnterpriseMore(eg, enterpriseIds)
		for k, v := range *list {
			v1 := v
			enterpriseName := ""
			_, ok := enterpriseMap[v.EnterpriseId]
			if ok {
				enterpriseName = enterpriseMap[v.EnterpriseId].Name
			}
			i := utils.IntToStr(k + 2)
			data["A"+i] = v1.BuyPhone
			data["B"+i] = enterpriseName
			data["C"+i] = utils.Int64ToStr(v1.Oid)
			data["D"+i] = v1.Amount
			data["E"+i] = stateArr[v1.State]
			data["F"+i] = gjson.Get(v1.BuyInfo, "date").String() + " " + gjson.Get(v1.BuyInfo, "time").String()
			data["G"+i] = v1.CreateAt.Format("2006-01-02 15:04:05")
		}
	}
	file := utils.Output(c, name, data)
	filename := name + ".xlsx"
	r := map[string]string{
		"file":     file,
		"filename": filename,
	}
	e.OutSuc(c, r, nil)
	return
}
func OrderEdit(c *gin.Context) {
	var args map[string]string
	if err := c.ShouldBindJSON(&args); err != nil {
		e.OutErr(c, e.ERR_INVALID_ARGS, err)
		return
	}
	eg := db.Db
	order := db.GetOrderById(eg, args["oid"])
	if order == nil {
		e.OutErr(c, 400, e.NewErr(400, "订单不存在"))
		return
	}
	var tmp md.BuyInfo
	json.Unmarshal([]byte(order.BuyInfo), &tmp)
	tmp.Time = args["time"]
	tmp.Date = args["date"]
	tmp.Phone = args["buy_phone"]
	tmp.Name = args["buy_name"]
	tmp.Address = args["address"]
	order.BuyPhone = args["buy_phone"]
	order.BuyInfo = utils.SerializeStr(tmp)
	eg.Where("id=?", order.Id).Cols("buy_phone,buy_info").Update(order)
	e.OutSuc(c, "success", nil)
	return
}
func OrderDel(c *gin.Context) {
	var args map[string]string
	if err := c.ShouldBindJSON(&args); err != nil {
		e.OutErr(c, e.ERR_INVALID_ARGS, err)
		return
	}
	eg := db.Db
	order := db.GetOrderById(eg, args["oid"])
	if order == nil {
		e.OutErr(c, 400, e.NewErr(400, "订单不存在"))
		return
	}
	sess := eg.NewSession()
	defer sess.Close()
	sess.Begin()
	orderGoodsSku := db.GetOrderGoodsById(eg, args["id"])
	i, err2 := eg.Where("id=?", args["id"]).Delete(&model.OrderGoods{})
	if i == 0 || err2 != nil {
		sess.Rollback()
		e.OutErr(c, 400, e.NewErr(400, "删除失败"))
		return
	}
	//销量扣除
	sql := `UPDATE goods SET sale=sale-%d WHERE id=%d`
	sql = fmt.Sprintf(sql, orderGoodsSku.Num, orderGoodsSku.GoodsId)
	db.QueryNativeStringSess(sess, sql)
	order.Amount = utils.Float64ToStr(utils.StrToFloat64(order.Amount) - float64(orderGoodsSku.Num)*utils.StrToFloat64(orderGoodsSku.Price))
	_, err := sess.Where("id=?", order.Id).Cols("amount").Update(order)
	if err != nil {
		sess.Rollback()
		e.OutErr(c, 400, e.NewErr(400, "修改失败"))
		return
	}
	sess.Commit()
	e.OutSuc(c, "success", nil)
	return
}
func OrderEditNum(c *gin.Context) {
	var args map[string]string
	if err := c.ShouldBindJSON(&args); err != nil {
		e.OutErr(c, e.ERR_INVALID_ARGS, err)
		return
	}
	eg := db.Db
	order := db.GetOrderById(eg, args["oid"])
	if order == nil {
		e.OutErr(c, 400, e.NewErr(400, "订单不存在"))
		return
	}
	order = JudgePackageOrdOrdState(order)
	if order.State == 5 {
		e.OutErr(c, 400, e.NewErr(400, "订单已取消,不能修改"))
		return
	}
	if order.State > 0 {
		e.OutErr(c, 400, e.NewErr(400, "订单制作中，不能修改"))
		return
	}
	sess := eg.NewSession()
	defer sess.Close()
	sess.Begin()

	orderGoods := db.GetOrderGoodsByIdSess(sess, args["id"])
	isHasSku := 0
	if orderGoods.SkuId != utils.StrToInt64(args["sku_id"]) {
		orderGoodsSku := db.GetOrderGoodsBySkuIdSess(sess, args["oid"], args["sku_id"])
		if orderGoodsSku != nil {
			isHasSku = 1
			//先删掉修改的 ，然后改另外的sku
			i, err := sess.Where("id=?", args["id"]).Delete(&model.OrderGoods{})
			if i == 0 || err != nil {
				sess.Rollback()
				e.OutErr(c, 400, e.NewErr(400, "修改失败"))
				return
			}
			//销量扣除
			sql := `UPDATE goods SET sale=sale-%d WHERE id=%d`
			sql = fmt.Sprintf(sql, orderGoods.Num, orderGoods.GoodsId)
			db.QueryNativeStringSess(sess, sql)
			//销量增加
			sqlSecond := `UPDATE goods SET sale=sale+%d WHERE id=%d`
			sqlSecond = fmt.Sprintf(sqlSecond, args["num"], orderGoodsSku.GoodsId)
			db.QueryNativeStringSess(sess, sqlSecond)
			//重新计算价格
			orderGoodsSku.Num += utils.StrToInt(args["num"])
			order.Amount = utils.Float64ToStr(utils.StrToFloat64(order.Amount) + utils.StrToFloat64(args["num"])*utils.StrToFloat64(orderGoodsSku.Price))
			_, err = sess.Where("id=?", orderGoodsSku.Id).Cols("num").Update(orderGoodsSku)
			if err != nil {
				sess.Rollback()
				e.OutErr(c, 400, e.NewErr(400, "修改失败"))
				return
			}
		} else {
			orderGoods.SkuId = utils.StrToInt64(args["sku_id"])
			_, sku, _ := db.GetMallSkuBySkuId(eg, args["sku_id"], utils.Int64ToStr(orderGoods.GoodsId))
			if sku != nil {
				orderGoods.SkuCode = sku.SkuCode
				orderGoods.Sku = sku.Sku
			}
		}
	}
	if isHasSku == 0 {
		num := utils.StrToInt(args["num"]) - orderGoods.Num
		if num < 0 { //少要了 就总金额 减少
			num = orderGoods.Num - utils.StrToInt(args["num"])
			order.Amount = utils.Float64ToStr(utils.StrToFloat64(order.Amount) - float64(num)*utils.StrToFloat64(orderGoods.Price))
			//销量扣除
			sqlSecond := `UPDATE goods SET sale=sale-%d WHERE id=%d`
			sqlSecond = fmt.Sprintf(sqlSecond, num, orderGoods.GoodsId)
			db.QueryNativeStringSess(sess, sqlSecond)
		} else {
			order.Amount = utils.Float64ToStr(utils.StrToFloat64(order.Amount) + float64(num)*utils.StrToFloat64(orderGoods.Price))
			//销量增加
			sqlSecond := `UPDATE goods SET sale=sale+%d WHERE id=%d`
			sqlSecond = fmt.Sprintf(sqlSecond, num, orderGoods.GoodsId)
			db.QueryNativeStringSess(sess, sqlSecond)
		}
		orderGoods.Num = utils.StrToInt(args["num"])
		_, err := sess.Where("id=?", orderGoods.Id).Cols("num,sku_code,sku_id,sku").Update(orderGoods)
		if err != nil {
			sess.Rollback()
			e.OutErr(c, 400, e.NewErr(400, "修改失败"))
			return
		}
	}
	_, err := sess.Where("id=?", order.Id).Cols("amount").Update(order)
	if err != nil {
		sess.Rollback()
		e.OutErr(c, 400, e.NewErr(400, "修改失败"))
		return
	}
	sess.Commit()
	e.OutSuc(c, "success", nil)
	return
}
